import 'dart:core';

/// Efficient prefix sums.
///
/// Based on: https://algs4.cs.princeton.edu/99misc/FenwickTree.java.html
abstract class MonoidalFenwickTree<T> {
  /// Increases the value at [index] by [value].
  void increase({
    required final int index,
    required final T value,
  });

  /// Returns the sum up to the given index [index_to].
  ///
  /// Example:
  /// If the tree is built with [1, 2, 3, 4, 5],
  /// then sum(4) will return the sum of [1, 2, 3, 4, 5] which is 15.
  T sum({
    required final int index_to,
  });

  /// Returns the amount of items that this contains.
  ///
  /// Example:
  /// If the tree is built with [1, 2, 3, 4, 5],
  /// then length() will return 5.
  int length();
}

/// A [MonoidalFenwickTree] that has an inverse operation.
abstract class GroupalFenwickTree<T> implements MonoidalFenwickTree<T> {
  /// Sets the value at [index] to be [value].
  void update({
    required final int index,
    required final T value,
  });

  /// Returns the sum between two indices.
  ///
  /// Example:
  /// If the tree is built with [1, 2, 3, 4, 5]
  /// then rangeSum(1, 4) will return the sum of [2, 3, 4] which is 9.
  T range_sum({
    required final int index_from,
    required final int index_to,
  });
}
